package com.flkr.gametest01;

import static org.anddev.andengine.extension.physics.box2d.util.constants.PhysicsConstants.PIXEL_TO_METER_RATIO_DEFAULT;

import java.util.ArrayList;

import org.anddev.andengine.engine.Engine;
import org.anddev.andengine.engine.camera.Camera;
import org.anddev.andengine.engine.options.EngineOptions;
import org.anddev.andengine.engine.options.EngineOptions.ScreenOrientation;
import org.anddev.andengine.engine.options.resolutionpolicy.RatioResolutionPolicy;
import org.anddev.andengine.entity.primitive.Rectangle;
import org.anddev.andengine.entity.scene.Scene;
import org.anddev.andengine.entity.scene.Scene.IOnSceneTouchListener;
import org.anddev.andengine.entity.scene.background.ColorBackground;
import org.anddev.andengine.entity.shape.Shape;
import org.anddev.andengine.entity.sprite.BaseSprite;
import org.anddev.andengine.extension.physics.box2d.PhysicsConnector;
import org.anddev.andengine.extension.physics.box2d.PhysicsConnectorManager;
import org.anddev.andengine.extension.physics.box2d.PhysicsFactory;
import org.anddev.andengine.extension.physics.box2d.PhysicsWorld;
import org.anddev.andengine.input.touch.TouchEvent;
import org.anddev.andengine.opengl.texture.Texture;
import org.anddev.andengine.opengl.texture.TextureOptions;
import org.anddev.andengine.opengl.texture.region.TextureRegionFactory;
import org.anddev.andengine.opengl.texture.region.TiledTextureRegion;
import org.anddev.andengine.sensor.accelerometer.AccelerometerData;
import org.anddev.andengine.sensor.accelerometer.IAccelerometerListener;
import org.anddev.andengine.ui.activity.BaseGameActivity;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.hardware.SensorManager;
import android.net.Uri;
import android.os.Bundle;
import android.text.Html;
import android.text.Spanned;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.widget.Toast;

import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.badlogic.gdx.physics.box2d.BodyDef.BodyType;
import com.flkr.gametest01.events.SpriteEvent;
import com.flkr.gametest01.events.SpriteEventConstants;
import com.flkr.gametest01.events.SpriteEventHandler;


public class GameTest01 extends BaseGameActivity implements IAccelerometerListener, IOnSceneTouchListener, SpriteEventHandler {

	// ===========================================================
	// Constants
	// ===========================================================

	private static final int CAMERA_WIDTH = 720;
	private static final int CAMERA_HEIGHT = 480;
	private static final int  WELCOME_DIALOG = 0;

	// ===========================================================
	// Fields
	// ===========================================================

	private Camera mCamera;
	private Texture mTexture;

	private TiledTextureRegion mBoxFaceTextureRegion;
	private TiledTextureRegion mCircleFaceTextureRegion;
	private TiledTextureRegion mTriangleFaceTextureRegion;
	private TiledTextureRegion mHexagonFaceTextureRegion;

	private PhysicsWorld mPhysicsWorld;

	private int mCharacterCount = 0;
	
	private BaseSprite secretCharacter;
	

	// ===========================================================
	// Constructors
	// ===========================================================

	// ===========================================================
	// Getter & Setter
	// ===========================================================

	// ===========================================================
	// Methods for/from SuperClass/Interfaces
	// ===========================================================

	
	
	
	public Engine onLoadEngine() {
		this.mCamera = new Camera(0, 0, CAMERA_WIDTH, CAMERA_HEIGHT);
		return new Engine(new EngineOptions(true, ScreenOrientation.LANDSCAPE, new RatioResolutionPolicy(CAMERA_WIDTH, CAMERA_HEIGHT), this.mCamera));
	}

	
	@Override
	protected void onCreate(Bundle pSavedInstanceState) {
		
		// TODO Auto-generated method stub
		super.onCreate(pSavedInstanceState);
		
		this.showDialog(WELCOME_DIALOG);
	}

	
	@Override
	protected Dialog onCreateDialog(int id) {
		// TODO Auto-generated method stub
		
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setMessage(R.string.app_description)
		       .setCancelable(false)
		       .setPositiveButton(R.string.btn_start, null)
		       .setNegativeButton("web ->", new DialogInterface.OnClickListener() {
		    	   public void onClick(DialogInterface dialog, int id) {
		   				//dialog.cancel();
		   		        GameTest01.this.startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse("http://pixelist.info")));
		           }
		       });
		AlertDialog alert = builder.create();
		
		return alert;
	}


	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.layout.about_menu, menu);

        //setContentView(R.layout.about_menu);
        
        return true;
    }

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
	    switch (item.getItemId()) {
	        case R.id.instructions: 
	        	Spanned marked_up = Html.fromHtml(GameTest01.this.getResources().getString(R.string.instructions_txt));
	        	
	        	Toast.makeText(GameTest01.this, marked_up.toString(), Toast.LENGTH_LONG).show();
	                            break;
	        case R.id.quit:     GameTest01.this.finish();
	                            break;
	    }
	    return true;
	}
    
	public void onLoadResources() {
		
		//Debug.d("Loading resources");
		
		this.mTexture = new Texture(64, 128, TextureOptions.BILINEAR);
		TextureRegionFactory.setAssetBasePath("gfx/");
		
		this.mBoxFaceTextureRegion = TextureRegionFactory.createTiledFromAsset(this.mTexture, this, "sq_tiled.png", 0, 0, 2, 1); // 64x32
		this.mCircleFaceTextureRegion = TextureRegionFactory.createTiledFromAsset(this.mTexture, this, "circ_tiled.png", 0, 32, 2, 1); // 64x32
		this.mTriangleFaceTextureRegion = TextureRegionFactory.createTiledFromAsset(this.mTexture, this, "face_triangle_tiled.png", 0, 64, 2, 1); // 64x32
		this.mHexagonFaceTextureRegion = TextureRegionFactory.createTiledFromAsset(this.mTexture, this, "face_hexagon_tiled.png", 0, 96, 2, 1); // 64x32
		this.mEngine.getTextureManager().loadTexture(this.mTexture);
		
		//Debug.d("Enable accelerometer");
		
		this.enableAccelerometerSensor(this);	
	}


	public Scene onLoadScene() {
		
		//Debug.d("Loading scene");
		
		//this.mEngine.registerUpdateHandler(new FPSLogger());
		
		final Scene scene = new Scene(2);
		scene.setBackground(new ColorBackground(0.9804f, 0f, 0.8784f));

		//Debug.d("set touch listener");
		
		scene.setOnSceneTouchListener(this);

		//Debug.d("create physics world");
		
		this.mPhysicsWorld = new PhysicsWorld(new Vector2(0, SensorManager.GRAVITY_EARTH), false);

		final Shape ground = new Rectangle(0, CAMERA_HEIGHT - 2, CAMERA_WIDTH, 2);
		final Shape roof = new Rectangle(0, 0, CAMERA_WIDTH, 2);
		final Shape left = new Rectangle(0, 0, 2, CAMERA_HEIGHT);
		final Shape right = new Rectangle(CAMERA_WIDTH - 2, 0, 2, CAMERA_HEIGHT);

		final FixtureDef wallFixtureDef = PhysicsFactory.createFixtureDef(0, 0.5f, 0.5f);
		PhysicsFactory.createBoxBody(this.mPhysicsWorld, ground, BodyType.StaticBody, wallFixtureDef);
		PhysicsFactory.createBoxBody(this.mPhysicsWorld, roof, BodyType.StaticBody, wallFixtureDef);
		PhysicsFactory.createBoxBody(this.mPhysicsWorld, left, BodyType.StaticBody, wallFixtureDef);
		PhysicsFactory.createBoxBody(this.mPhysicsWorld, right, BodyType.StaticBody, wallFixtureDef);

		scene.getBottomLayer().addEntity(ground);
		scene.getBottomLayer().addEntity(roof);
		scene.getBottomLayer().addEntity(left);
		scene.getBottomLayer().addEntity(right);

		scene.registerUpdateHandler(this.mPhysicsWorld);
	
		return scene;
	}


	public void onLoadComplete() {
		//Debug.d("scene load complete");
	}

	@Override
	public boolean onSceneTouchEvent(final Scene pScene, final TouchEvent pSceneTouchEvent) {
		if(this.mPhysicsWorld != null) {
			if(pSceneTouchEvent.getAction() == MotionEvent.ACTION_DOWN) {
				this.runOnUpdateThread(new Runnable() {
					@Override
					public void run() {
						addCharacter(pSceneTouchEvent.getX(), pSceneTouchEvent.getY());
					}
				});
				return true;
			}
		}
		return false;
	}

	@Override
	public void onAccelerometerChanged(final AccelerometerData pAccelerometerData) {
		this.mPhysicsWorld.setGravity(new Vector2(pAccelerometerData.getY(), pAccelerometerData.getX()));
	}
	
	@Override
	public boolean spriteEvent(final SpriteEvent _spriteEvent)
	{
		boolean result = false;
		
		switch(_spriteEvent.mSpriteEvent)
		{
		case SpriteEventConstants.SPRITE_CLICKED:
			if (_spriteEvent.mSprite.isValid)
			{
				_spriteEvent.mSprite.isValid = false;
				PhysicsConnectorManager pcm = this.mPhysicsWorld.getPhysicsConnectorManager();
				
				final Body _dead = pcm.findBodyByShape(_spriteEvent.mSprite);
				
				final ArrayList<PhysicsConnector> physicsConnectors = pcm;
				int i=0;
				boolean found = false;
				while(i<physicsConnectors.size() && !found) 
				{
					final PhysicsConnector physicsConnector = physicsConnectors.get(i);
					if(physicsConnector.getBody() == _dead)
					{
						found = true;
						this.mPhysicsWorld.unregisterPhysicsConnector(physicsConnector);
						//this.mPhysicsWorld.destroyBody(_dead);
						
						final Scene scene = this.mEngine.getScene();
						
						this.runOnUpdateThread(new Runnable() {
							@Override
							public void run() {
								mPhysicsWorld.destroyBody(_dead);
								scene.getLayer(1).removeEntity(_spriteEvent.mSprite);
							}
						});
											
						result = true;
						
					}
					else
						i++;
				}
			}
		}
		return result;
	}
	
	
	// ===========================================================
	// Methods
	// ===========================================================
	
	
	private void addCharacter(final float pX, final float pY) {
		final Scene scene = this.mEngine.getScene();

		//Debug.d("Faces: " + this.mCharacterCount);

		final TouchSprite character;
		final Body body;

		final FixtureDef objectFixtureDef = PhysicsFactory.createFixtureDef(1, 0.5f, 0.5f);

		if(this.mCharacterCount % 2 == 0) {
			character = new TouchSprite(pX, pY, 64, 64, this.mBoxFaceTextureRegion.clone());
			body = PhysicsFactory.createBoxBody(this.mPhysicsWorld, character, BodyType.DynamicBody, objectFixtureDef);
		} else {
			character = new TouchSprite(pX, pY, 64, 64, this.mCircleFaceTextureRegion.clone());
			body = PhysicsFactory.createCircleBody(this.mPhysicsWorld, character, BodyType.DynamicBody, objectFixtureDef);
		} 
//		else if (this.mCharacterCount % 4 == 2) {
//			character = new TouchSprite(pX, pY, this.mTriangleFaceTextureRegion);
//			body = createTriangleBody(this.mPhysicsWorld, character, BodyType.DynamicBody, objectFixtureDef);
//		} else {
//			character = new TouchSprite(pX, pY, this.mHexagonFaceTextureRegion);
//			body = createHexagonBody(this.mPhysicsWorld, character, BodyType.DynamicBody, objectFixtureDef);
//		}

		character.setSpriteEventHandler((SpriteEventHandler)this);
		character.animate(100);
		character.setUpdatePhysics(true);
		scene.registerTouchArea(character);
		scene.getTopLayer().addEntity(character);
		this.mPhysicsWorld.registerPhysicsConnector(new PhysicsConnector(character, body, true, true, false, false));
		
		if (++this.mCharacterCount < 2) secretCharacter = (BaseSprite)character;
	}

	/**
	 * Creates a {@link Body} based on a {@link PolygonShape} in the form of a triangle:
	 * <pre>
	 *  /\
	 * /__\
	 * </pre>
	 */
	public static Body createTriangleBody(final PhysicsWorld pPhysicsWorld, final Shape pShape, final BodyType pBodyType, final FixtureDef pFixtureDef) {
		/* Remember that the vertices are relative to the center-coordinates of the Shape. */
		final float halfWidth = pShape.getWidthScaled() * 0.5f / PIXEL_TO_METER_RATIO_DEFAULT;
		final float halfHeight = pShape.getHeightScaled() * 0.5f / PIXEL_TO_METER_RATIO_DEFAULT;

		final float top = -halfHeight;
		final float bottom = halfHeight;
		final float left = -halfHeight;
		final float centerX = 0;
		final float right = halfWidth;

		final Vector2[] vertices = {
				new Vector2(centerX, top),
				new Vector2(right, bottom),
				new Vector2(left, bottom)
		};

		return PhysicsFactory.createPolygonBody(pPhysicsWorld, pShape, vertices, pBodyType, pFixtureDef);
	}

	/**
	 * Creates a {@link Body} based on a {@link PolygonShape} in the form of a hexagon:
	 * <pre>
	 *  /\
	 * /  \
	 * |  |
	 * |  |
	 * \  /
	 *  \/
	 * </pre>
	 */
	public static Body createHexagonBody(final PhysicsWorld pPhysicsWorld, final Shape pShape, final BodyType pBodyType, final FixtureDef pFixtureDef) {
		/* Remember that the vertices are relative to the center-coordinates of the Shape. */
		final float halfWidth = pShape.getWidthScaled() * 0.5f / PIXEL_TO_METER_RATIO_DEFAULT;
		final float halfHeight = pShape.getHeightScaled() * 0.5f / PIXEL_TO_METER_RATIO_DEFAULT;

		/* The top and bottom vertex of the hexagon are on the bottom and top of hexagon-sprite. */
		final float top = -halfHeight;
		final float bottom = halfHeight;

		final float centerX = 0;

		/* The left and right vertices of the heaxgon are not on the edge of the hexagon-sprite, so we need to inset them a little. */
		final float left = -halfWidth + 2.5f / PIXEL_TO_METER_RATIO_DEFAULT;
		final float right = halfWidth - 2.5f / PIXEL_TO_METER_RATIO_DEFAULT;
		final float higher = top + 8.25f / PIXEL_TO_METER_RATIO_DEFAULT;
		final float lower = bottom - 8.25f / PIXEL_TO_METER_RATIO_DEFAULT;

		final Vector2[] vertices = {
				new Vector2(centerX, top),
				new Vector2(right, higher),
				new Vector2(right, lower),
				new Vector2(centerX, bottom),
				new Vector2(left, lower),
				new Vector2(left, higher)
		};

		return PhysicsFactory.createPolygonBody(pPhysicsWorld, pShape, vertices, pBodyType, pFixtureDef);
	}
	
	// ===========================================================
	// Inner and Anonymous Classes
	// ===========================================================
}